@@ -25,7 +25,7 @@ typedef struct fcs_derived_state_struct
 {
     fcs_state_keyval_pair state;
     fcs_encoded_state_buffer key;
-    fcs_dbm_record *parent;
+    fcs_dbm_store_val parent;
     struct fcs_derived_state_struct *next;
     size_t core_irreversible_moves_count;
     fcs_which_moves_bitmask which_irreversible_moves_bitmask;
@@ -326,7 +326,7 @@ static inline bool is_state_solved(
 static inline bool instance_solver_thread_calc_derived_states(
     const fcs_dbm_variant_type local_variant,
     fcs_state_keyval_pair *const init_state_kv_ptr,
-    fcs_dbm_record *const parent_ptr, fcs_derived_state **const derived_list,
+    fcs_dbm_store_val parent_ptr, fcs_derived_state **const derived_list,
     fcs_derived_state **const derived_list_recycle_bin,
     compact_allocator *const derived_list_allocator,
     const bool perform_horne_prune)
