@@ -50,7 +50,6 @@ static void print_int(fcs_binary_output *const bin, int val)
     if (bin->ptr == bin->buffer_end)
     {
         write_me(bin);
-        /* Reset ptr to the beginning */
         bin->ptr = bin->buffer;
     }
 }
@@ -89,7 +88,8 @@ static void read_int_wrapper(FILE *const in, long long *const var)
 }
 
 static inline void bin_init(fcs_binary_output *const bin,
-    long long *const start_board_ptr, long long *const end_board_ptr,
+    fc_solve_ms_deal_idx_type *const start_board_ptr,
+    fc_solve_ms_deal_idx_type *const end_board_ptr,
     fcs_int_limit_t *const total_iterations_limit_per_board_ptr)
 {
     if (bin->filename)
@@ -113,8 +113,12 @@ static inline void bin_init(fcs_binary_output *const bin,
         }
         else
         {
-            read_int_wrapper(in, start_board_ptr);
-            read_int_wrapper(in, end_board_ptr);
+            long long temp = 0;
+            read_int_wrapper(in, &temp);
+            *start_board_ptr = (fc_solve_ms_deal_idx_type)temp;
+            temp = 0;
+            read_int_wrapper(in, &temp);
+            *end_board_ptr = (fc_solve_ms_deal_idx_type)temp;
             {
                 long long val;
                 read_int_wrapper(in, &val);
@@ -128,7 +132,8 @@ static inline void bin_init(fcs_binary_output *const bin,
                 exit_error(
                     "%s", "Output file has an invalid length. Terminating.\n");
             }
-            *start_board_ptr += (file_len - 12) / 4;
+            *start_board_ptr +=
+                (fc_solve_ms_deal_idx_type)((file_len - 12) / 4);
             if (*start_board_ptr >= *end_board_ptr)
             {
                 exit_error("%s",
