@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use autodie;
 
-use FindBin;
+use FindBin ();
 use lib "$FindBin::Bin/../t/lib";
 
 use FC_Solve::SplitTests ();
