@@ -10,6 +10,10 @@
 // reduce the memory consumption. See
 // ../docs/split-fully-connected-components-based-solver-planning.txt
 // in the Freecell Solver git repository.
+#ifndef FCS_DBM__VAL_IS_ANCESTOR
+#error FCS_DBM__VAL_IS_ANCESTOR must be defined
+#endif
+
 #include "dbm_solver_head.h"
 #include <sys/tree.h>
 #include "depth_multi_queue.h"
@@ -343,7 +347,13 @@ static void *instance_run_solver_thread(void *const void_arg)
 #endif
 
             if (instance_solver_thread_calc_derived_states(local_variant,
-                    &state, token, &derived_list, &derived_list_recycle_bin,
+                    &state,
+#ifdef FCS_DBM__VAL_IS_ANCESTOR
+                    token->ancestor,
+#else
+                    token,
+#endif
+                    &derived_list, &derived_list_recycle_bin,
                     &derived_list_allocator, true))
             {
                 fcs_lock_lock(&instance->global_lock);
@@ -370,6 +380,7 @@ static void *instance_run_solver_thread(void *const void_arg)
 
             fcs_derived_state_list__recycle(
                 &derived_list_recycle_bin, &derived_list);
+            derived_list = NULL;
             // End handle item.
         }
         // End of main thread loop
@@ -400,7 +411,7 @@ static inline void instance_alloc_num_moves(
 static inline void instance_check_key(
     dbm_solver_thread *const thread, dbm_solver_instance *const instance,
     const size_t key_depth, fcs_encoded_state_buffer *const key,
-    fcs_dbm_record *const parent, const unsigned char move GCC_UNUSED,
+    fcs_dbm_store_val parent, const unsigned char move GCC_UNUSED,
     const fcs_which_moves_bitmask *const which_irreversible_moves_bitmask
 #ifndef FCS_DBM_WITHOUT_CACHES
     ,
@@ -421,7 +432,9 @@ static inline void instance_check_key(
         cache_store__insert_key(
             &(coll->cache_store), key, parent, moves_to_parent, move);
 #endif
-
+#if 0
+        printf("key_depth = %zu ; curr_depth = %zu\n", key_depth, instance->curr_depth);
+#endif
         if (key_depth == instance->curr_depth)
         {
             // Now insert it into the queue.
@@ -450,17 +463,22 @@ static inline void instance_check_key(
                     which_irreversible_moves_bitmask->s[i] +
                     instance->fingerprint_which_irreversible_moves_bitmask.s[i];
             }
-            size_t trace_num;
-            fcs_encoded_state_buffer *trace;
             fcs_lock_lock(&instance->fcc_exit_points_output_lock);
             // instance->storage_lock is already locked in
             // instance_check_multiple_keys and we should not lock it here.
+#ifndef FCS_DBM__VAL_IS_ANCESTOR
+            size_t trace_num;
+            fcs_encoded_state_buffer *trace;
             calc_trace(token, &trace, &trace_num);
+#endif
             {
+#ifndef FCS_DBM__VAL_IS_ANCESTOR
                 FccEntryPointNode fcc_entry_key;
                 fcc_entry_key.kv.key.key = trace[trace_num - 1];
                 FccEntryPointNode *val_proto = RB_FIND(FccEntryPointList,
-                    &(instance->fcc_entry_points), &fcc_entry_key);
+#else
+                FccEntryPointNode *val_proto = token->ancestor;
+#endif
                 if (!val_proto)
                 {
                     goto cleanup;
@@ -499,6 +517,15 @@ static inline void instance_check_key(
                     &(instance->moves_to_state_len));
             }
 
+#ifdef FCS_DBM__VAL_IS_ANCESTOR
+            const fcs_encoded_state_buffer outkey = token->key;
+            const size_t keysize = sizeof(outkey);
+            const_SLOT(moves_to_state_len, instance);
+            const size_t added_moves_to_output = moves_to_state_len + keysize;
+            instance_alloc_num_moves(instance, added_moves_to_output);
+            unsigned char *const moves_to_state = instance->moves_to_state;
+            memcpy(&moves_to_state[moves_to_state_len], &outkey, keysize);
+#else
             const_SLOT(moves_to_state_len, instance);
             const size_t added_moves_to_output =
                 moves_to_state_len + trace_num - 1;
@@ -512,6 +539,7 @@ static inline void instance_check_key(
                         &(thread->delta_stater), trace[i], trace[i - 1]);
             }
 
+#endif
             const size_t new_max_enc_len =
                 ((added_moves_to_output * 4) / 3) + 20;
 
@@ -555,7 +583,9 @@ static inline void instance_check_key(
             fflush(instance->fcc_exit_points_out_fh);
         cleanup:
             fcs_lock_unlock(&instance->fcc_exit_points_output_lock);
+#ifndef FCS_DBM__VAL_IS_ANCESTOR
             free(trace);
+#endif
         }
     }
 }
@@ -706,7 +736,11 @@ int main(int argc, char *argv[])
     {
         FccEntryPointNode *const entry_point = fcs_compact_alloc_ptr(
             &(instance.fcc_entry_points_allocator), sizeof(*entry_point));
+#ifdef FCS_DBM__VAL_IS_ANCESTOR
+        entry_point->kv.key.ancestor = entry_point;
+#else
         fcs_dbm_record_set_parent_ptr(&(entry_point->kv.key), NULL);
+#endif
         char state_base64[100];
         int state_depth;
         sscanf(
