@@ -7,8 +7,8 @@ use 5.014;
 
 use parent 'FC_Solve::DeltaStater::DeBondt';
 
-use Carp ();
-use List::Util qw/ first max /;
+use Carp       ();
+use List::Util qw/ any first max uniq /;
 
 use Games::Solitaire::Verify::Card        ();
 use Games::Solitaire::Verify::Foundations ();
@@ -28,6 +28,7 @@ See git/fc-solve/fc-solve/docs/debondt-compact-freecell-positions--document.asci
 __PACKAGE__->mk_acc_ref( [qw(_proxied_worker)] );
 
 use FC_Solve::DeltaStater::Constants qw/
+    $NUM_KING_OPTS
     $NUM_OPTS
     $OPT_DONT_CARE
     $OPT_FREECELL
@@ -46,12 +47,17 @@ my $INFERRED_SUB_STATE               = -1;
 sub _assert_def
 {
     my $val = shift;
+    if ( $val < 0 )
+    {
+        Carp::confess("negative");
+    }
     return $val // do { Carp::confess("undefined"); };
 }
 
 package FC_Solve::DeltaStater::FccFingerPrint::StatesRecord;
 
 use parent 'Games::Solitaire::Verify::Base';
+use List::Util qw/ any first max uniq /;
 
 __PACKAGE__->mk_acc_ref(
     [
@@ -101,8 +107,12 @@ sub _add_state_pair
         $rec->CARD_PAIR_STATES_MAP()->[$y]->[$x] = $val;
         $rec->REVERSE_CARD_PAIR_STATES_MAP()->[$val] = [
             map {
-                FC_Solve::DeltaStater::FccFingerPrint::StatesRecord::_string_to_int(
-                    $_)
+                1
+                    ? $rec->single_card_states->{$_}
+                    : (
+                    FC_Solve::DeltaStater::FccFingerPrint::StatesRecord::_string_to_int(
+                        $_)
+                    )
             } @input
         ];
     }
@@ -110,6 +120,16 @@ sub _add_state_pair
 
 }
 
+sub _assert_def
+{
+    my $val = shift;
+    if ( $val < 0 )
+    {
+        Carp::confess("negative");
+    }
+    return $val // do { Carp::confess("undefined"); };
+}
+
 sub finalize
 {
     my $rec = shift;
@@ -139,6 +159,7 @@ sub finalize
     $_add->( 'PARENT_0',       'PARENT_1' );
     $_add->( 'PARENT_1',       'PARENT_0' );
     $rec->CARD_PAIR_STATE_BASE( $rec->state_opt_next() );
+    _assert_def( $rec->rev_single_card_states->[1] );
     return;
 }
 
@@ -175,6 +196,62 @@ sub _my_add_state
     return;
 }
 
+sub _calc_initial_possible_pair_states
+{
+    my ($rec) = @_;
+    my $variant_states = $rec;
+    return {
+        map {
+            my $pair = $_;
+            $pair => +{
+                map {
+                    my $rank   = $_;
+                    my $values = [ 0 .. ( $rec->CARD_PAIR_STATE_BASE() - 1 ) ];
+                    if ( $rank == $RANK_KING )
+                    {
+                        @$values = (
+                            grep {
+                                my $i = $_;
+                                not any
+                                {
+                                    $_ == $variant_states->single_card_states()
+                                        ->{'PARENT_0'}
+                                        or $_ ==
+                                        $variant_states->single_card_states()
+                                        ->{'PARENT_1'}
+                                    } @{ $rec->REVERSE_CARD_PAIR_STATES_MAP
+                                        ->[$i] }
+                            } @$values
+                        );
+
+                        # say "values = @$values";
+                    }
+                    $rank => $values
+                } ( 1 .. $RANK_KING )
+            };
+        } 0 .. 1
+    };
+}
+
+sub _calc__remaining__single_options
+{
+    my ( $variant_states, $card_idx, $options ) = @_;
+
+    my @single_options = sort { $a <=> $b } uniq(
+        map {
+            my $xx = $variant_states->REVERSE_CARD_PAIR_STATES_MAP()->[$_]
+                ->[$card_idx];
+
+            $xx
+        } @$options
+    );
+    if ( @single_options > $variant_states->num_single_card_states )
+    {
+        die;
+    }
+    return \@single_options;
+}
+
 package FC_Solve::DeltaStater::FccFingerPrint;
 
 my $positive_rec = FC_Solve::DeltaStater::FccFingerPrint::StatesRecord->new(
@@ -219,25 +296,6 @@ sub _init
     return;
 }
 
-my $zero_fc_variant =
-    Games::Solitaire::Verify::VariantsMap->new->get_variant_by_id('freecell');
-
-$zero_fc_variant->num_freecells(0);
-
-sub _calc_state_obj_generic
-{
-    my ( $self, $args ) = @_;
-    return $self->_is_bakers_dozen()
-        ? ( die "unimpl" )
-        : Games::Solitaire::Verify::State->new(
-        {
-            variant        => 'custom',
-            variant_params => $zero_fc_variant,
-            %{$args},
-        },
-        );
-}
-
 sub _initialize_card_states
 {
     my ( $self, $num_opts ) = @_;
@@ -277,6 +335,17 @@ sub _init
 
 package FC_Solve::DeltaStater::FccFingerPrint;
 
+__PACKAGE__->mk_acc_ref(
+    [
+        qw[
+            _color
+            _rank
+            _possible_pair_states
+            _variant_states
+        ]
+    ]
+);
+
 sub _set_opt
 {
     my ( $self, $ref, $opt ) = @_;
@@ -288,19 +357,49 @@ sub _set_opt
     return;
 }
 
+use List::MoreUtils qw/ firstidx /;
+
 sub _encode_single_uknown_info_card
 {
-    my ( $self, $state_writer, $opt, $is_king, $variant_states ) = @_;
-    my $state_o = $opt->sub_state;
+    my ( $self, $card_idx, $state_writer, $opt, $is_king, $variant_states ) =
+        @_;
+    my $color                 = $self->_color;
+    my $rank                  = $self->_rank;
+    my $_possible_pair_states = $self->_possible_pair_states();
+    my @options               = @{ $_possible_pair_states->{$color}{$rank} };
+    my $single_options =
+        $variant_states->_calc__remaining__single_options( $card_idx,
+        \@options );
+    $self->_set_card_opt( $color, $rank, $opt->sub_state );
     if ($is_king)
     {
-        $state_writer->write( { base => 2, item => $state_o } );
+        if (
+            ( !$variant_states->does_have_zero_freecells() )
+            and (
+                any
+                {
+                    $_ == $variant_states->single_card_states()
+                        ->{'ABOVE_FREECELL'}
+                } @$single_options
+            )
+            )
+        {
+            my $state_o = _assert_def(
+                scalar firstidx { $_ == $opt->sub_state }
+                @$single_options
+            );
+            $state_writer->write( { base => 2, item => $state_o } );
+        }
     }
     else
     {
+        my $state_o = _assert_def(
+            scalar firstidx { $_ == $opt->sub_state }
+            @$single_options
+        );
         $state_writer->write(
             {
-                base => $variant_states->num_single_card_states(),
+                base => ( scalar @$single_options ),
                 item => $state_o
             }
         );
@@ -311,23 +410,52 @@ sub _encode_single_uknown_info_card
 sub _encode_a_pair_of_uknown_info_cards
 {
     my ( $self, $state_writer, $opt1, $opt2, $is_king, $variant_states ) = @_;
-    my @states = ( $opt1->sub_state, $opt2->sub_state );
+    my @states                = ( $opt1->sub_state, $opt2->sub_state );
+    my $color                 = $self->_color;
+    my $rank                  = $self->_rank;
+    my $_possible_pair_states = $self->_possible_pair_states();
+    foreach my $s (@states)
+    {
+        $self->_set_card_opt( $color, $rank, $s );
+    }
+    my @options = @{ $_possible_pair_states->{$color}{$rank} };
+    if ( @options > $variant_states->CARD_PAIR_STATE_BASE )
+    {
+        die;
+    }
     if ($is_king)
     {
-        foreach my $state_o (@states)
+        while ( my ( $card_idx, $state_o ) = each(@states) )
         {
-            $state_writer->write( { base => 2, item => $state_o } );
+            if (
+                ( !$variant_states->does_have_zero_freecells() )
+                and (
+                    any
+                    {
+                        my $xx = $variant_states->REVERSE_CARD_PAIR_STATES_MAP()
+                            ->[$_]->[$card_idx];
+                        $xx == $variant_states->single_card_states()
+                            ->{'ABOVE_FREECELL'}
+                    } @options
+                )
+                )
+            {
+                $state_writer->write( { base => 2, item => $state_o } );
+            }
         }
     }
     else
     {
+        my $state_o = firstidx
+        {
+            $_ == $variant_states->CARD_PAIR_STATES_MAP()->[ $states[0] ]
+                ->[ $states[1] ]
+        }
+        @options;
         $state_writer->write(
             {
-                base => _assert_def( $variant_states->CARD_PAIR_STATE_BASE() ),
-                item => _assert_def(
-                    $variant_states->CARD_PAIR_STATES_MAP()->[ $states[0] ]
-                        ->[ $states[1] ]
-                ),
+                base => _assert_def( scalar(@options) ),
+                item => _assert_def($state_o),
             }
         );
     }
@@ -347,7 +475,7 @@ sub _encode_cards_pair
         }
         else
         {
-            $self->_encode_single_uknown_info_card( $state_writer,
+            $self->_encode_single_uknown_info_card( 1, $state_writer,
                 $opt2, $is_king, $variant_states );
         }
     }
@@ -355,7 +483,7 @@ sub _encode_cards_pair
     {
         if ( $fingerprint2 != $ABOVE_PARENT_CARD_OR_EMPTY_SPACE )
         {
-            $self->_encode_single_uknown_info_card( $state_writer,
+            $self->_encode_single_uknown_info_card( 0, $state_writer,
                 $opt1, $is_king, $variant_states );
         }
         else
@@ -424,6 +552,9 @@ sub encode_composite
     my $variant_states      = $self->_calc_variant_states();
     my $should_skip_is_king = $variant_states->should_skip_is_king();
 
+    my $_possible_pair_states = $self->_possible_pair_states(
+        $variant_states->_calc_initial_possible_pair_states() );
+
     $self->_initialize_card_states_for_encode();
 
     my $state_writer       = FC_Solve::VarBaseDigitsWriter::XS->new;
@@ -441,6 +572,7 @@ sub encode_composite
             my $o =
                 $self->_calc_encoded_OptRecord( $col, $height, $card,
                 $variant_states );
+
             $self->_mark_suit_rank_as_true( $self->_get_suit_idx($card),
                 $card->rank, $o );
         }
@@ -470,8 +602,10 @@ sub encode_composite
     my $cnt = 0;
     foreach my $rank ( 1 .. $RANK_KING )
     {
+        $self->_rank($rank);
         foreach my $color ( 0 .. 1 )
         {
+            $self->_color($color);
             my $suit1 = $color;
             my $suit2 = ( $color | 2 );
             my $rank1 = $derived->get_foundation_value( $suits[$suit1], 0 );
@@ -498,7 +632,7 @@ sub encode_composite
                 : ${ $self->_opt_by_suit_rank( $suit2, $rank ) };
             die if !( defined $opt1->fingerprint_state );
 
-            if ( !( defined $opt2->fingerprint_state ) )
+            if ( ( !defined $opt2 ) or !( defined $opt2->fingerprint_state ) )
             {
                 $DB::single = 1;
                 Carp::confess("opt2");
@@ -513,8 +647,6 @@ sub encode_composite
                     $opt1, $opt2, $is_king, $variant_states );
             }
 
-            # print("$rank $color $fingerprint1 $fingerprint2\n");
-
             $fingerprint_writer->write( { base => 3, item => $fingerprint1 } );
             $fingerprint_writer->write( { base => 3, item => $fingerprint2 } );
             $cnt += 2;
@@ -552,7 +684,11 @@ sub encode_composite
 
 sub _calc_variant_states
 {
-    return shift()->_init_state->num_freecells ? $positive_rec : $zero_rec;
+    my ($self) = @_;
+
+    # Carp::confess( "numfc = " . $self->_init_state->num_freecells );
+    return $self->_variant_states(
+        $self->_init_state->num_freecells ? $positive_rec : $zero_rec );
 }
 
 my @SUIT_INDEXES =
@@ -561,6 +697,42 @@ my @SUIT_INDEXES =
 my $CARD_STATES_SKIP              = -1;
 my $CARD_STATES__TO_BE_DETERMINED = -2;
 
+sub _set_card_opt
+{
+    my ( $self, $color, $rank, $opt ) = @_;
+    if ( $rank == $RANK_KING )
+    {
+        return $opt;
+    }
+    my $variant_states        = $self->_variant_states;
+    my $_possible_pair_states = $self->_possible_pair_states();
+    if ( $variant_states->does_have_zero_freecells() )
+    {
+        return $opt;
+    }
+    my $parentrank = $rank + 1;
+    my $card_idx   = (
+        _assert_def($opt) ==
+            _assert_def( $variant_states->single_card_states->{'PARENT_1'} )
+        ? 1
+        : _assert_def($opt) ==
+            _assert_def( $variant_states->single_card_states->{'PARENT_0'} ) ? 0
+        : ( return $opt )
+    );
+    my $aref   = $_possible_pair_states->{ 0b1 ^ $color }{$parentrank};
+    my @before = @$aref;
+    @$aref = (
+        grep {
+            _assert_def(
+                $variant_states->REVERSE_CARD_PAIR_STATES_MAP()->[$_]
+                    ->[$card_idx] ) != _assert_def(
+                $variant_states->single_card_states->{'ABOVE_FREECELL'} )
+        } @before
+    );
+
+    return $opt;
+}
+
 sub decode
 {
     my ( $self, $encoded ) = @_;
@@ -593,6 +765,8 @@ sub decode
     my @queue;
     my @fingerprints;
     my @card_states;
+    my $_possible_pair_states = $self->_possible_pair_states(
+        $variant_states->_calc_initial_possible_pair_states() );
 
     my $foundations_obj = Games::Solitaire::Verify::Foundations->new(
         {
@@ -602,7 +776,7 @@ sub decode
     foreach my $rank ( 1 .. $RANK_KING )
     {
         my $is_king = ( $rank == $RANK_KING );
-        foreach my $indexes (@SUIT_INDEXES)
+        while ( my ( $color, $indexes ) = each(@SUIT_INDEXES) )
         {
             my @are_not_set;
             foreach my $suit_idx (@$indexes)
@@ -612,7 +786,6 @@ sub decode
                 my $newval;
                 if ( $f == $IN_FOUNDATIONS )
                 {
-                    # say "ff=$f $suit_idx";
                     $foundations_obj->assign( $suits[$suit_idx], 0, $rank );
                     $newval = $OPT_DONT_CARE;
                 }
@@ -641,38 +814,96 @@ sub decode
             }
             if ( $are_not_set[0] or $are_not_set[1] )
             {
-                if ( $are_not_set[0] and $are_not_set[1] )
+                if ($is_king)
                 {
-                    my $s = $state_reader->read(
-                        _assert_def( $variant_states->CARD_PAIR_STATE_BASE() )
-                    );
-                    my @pos = @{
-                        _assert_def(
-                            $variant_states->REVERSE_CARD_PAIR_STATES_MAP()
-                                ->[$s]
-                        )
-                    };
-                    for my $i ( keys @pos )
+                    my @options = @{ $_possible_pair_states->{$color}{$rank} };
+                KINGS:
+                    while ( my ( $card_idx, $state_o ) = each(@are_not_set) )
                     {
-                        $card_states[ $indexes->[$i] ][$rank] = $pos[$i];
+                        if ( !$state_o )
+                        {
+                            next KINGS;
+                        }
+                        my $data = 0;
+                        if (
+                            ( !$variant_states->does_have_zero_freecells() )
+                            and (
+                                any
+                                {
+                                    my $xx =
+                                        $variant_states
+                                        ->REVERSE_CARD_PAIR_STATES_MAP()->[$_]
+                                        ->[$card_idx];
+                                    $xx == $variant_states->single_card_states()
+                                        ->{'ABOVE_FREECELL'}
+                                } @options
+                            )
+                            )
+                        {
+                            $data = $state_reader->read(2);
+                        }
+                        my $single_options =
+                            $variant_states->_calc__remaining__single_options(
+                            $card_idx, \@options );
+                        Carp::confess("outofbound $data [ @$single_options ]")
+                            if $data >= @$single_options;
+                        my $oval = _assert_def(
+                            $variant_states->rev_single_card_states->[
+                                $single_options->[$data]
+                            ]
+                        );
+                        die "big $oval >= $NUM_KING_OPTS"
+                            if $oval >= $NUM_KING_OPTS;
+                        $card_states[ $indexes->[$card_idx] ][$rank] = $oval;
                     }
                 }
                 else
                 {
-                    my $s = $state_reader->read(
-                        _assert_def(
-                            $variant_states->num_single_card_states()
-                        )
-                    );
-                    my $pos =
-                        _assert_def(
-                        $variant_states->rev_single_card_states()->[$s] );
-                    $card_states[
-                        $indexes->[
-                        first { $are_not_set[$_] }
-                    0 .. 1
-                        ]
-                    ][$rank] = $pos;
+                    my @options =
+                        @{ $_possible_pair_states->{$color}{$rank} };
+                    my $set = sub {
+                        my ($opt) = @_;
+                        my $v = $self->_set_card_opt( $color, $rank, $opt );
+                        return _assert_def(
+                            $variant_states->rev_single_card_states->[$v] );
+                    };
+                    if ( $are_not_set[0] and $are_not_set[1] )
+                    {
+                        my $s =
+                            $options[ $state_reader->read( scalar(@options) ) ];
+                        my @pos = @{
+                            _assert_def(
+                                $variant_states->REVERSE_CARD_PAIR_STATES_MAP()
+                                    ->[$s]
+                            )
+                        };
+                        for my $i ( keys @pos )
+                        {
+                            $card_states[ $indexes->[$i] ][$rank] =
+                                $set->( $pos[$i] );
+                        }
+                    }
+                    else
+                    {
+                        my $card_idx = _assert_def(
+                            scalar( first { $are_not_set[$_] } 0 .. 1 ) );
+                        my @single_options = sort { $a <=> $b } uniq(
+                            map {
+                                _assert_def(
+                                    $variant_states
+                                        ->REVERSE_CARD_PAIR_STATES_MAP(
+                                    )->[$_]->[$card_idx]
+                                )
+                            } @options
+                        );
+                        my $s = $single_options[ $state_reader->read(
+                                scalar @single_options ) ];
+
+                        my $pos = _assert_def($s);
+
+                        $card_states[ $indexes->[$card_idx] ][$rank] =
+                            $set->($pos);
+                    }
                 }
             }
         }
@@ -680,10 +911,12 @@ sub decode
 
     foreach my $suit_idx ( 0 .. $#suits )
     {
-        push @queue, $foundations_obj->value( $suits[$suit_idx], 0 );
+        push @queue,
+            [
+            14, _assert_def( $foundations_obj->value( $suits[$suit_idx], 0 ) )
+            ];
     }
 
-    # say "@queue";
     foreach my $rank ( 1 .. $self->_get_top_rank_for_iter() )
     {
         foreach my $suit_idx ( 0 .. $#suits )
@@ -691,7 +924,13 @@ sub decode
             my $val = $card_states[$suit_idx][$rank];
             if ( $val >= 0 )
             {
-                push @queue, $val;
+                my $base = $NUM_OPTS;
+                if ( $rank == $RANK_KING )
+                {
+                    $base = $NUM_KING_OPTS;
+                    die "big $val >= $NUM_KING_OPTS" if $val >= $NUM_KING_OPTS;
+                }
+                push @queue, [ _assert_def($base), _assert_def($val) ];
             }
             else
             {
@@ -747,12 +986,17 @@ sub _init
 sub read
 {
     my ( $self, $base ) = @_;
+    if ( ref($base) ne "" or ( $base != int($base) ) )
+    {
+        Carp::confess("wrong $base");
+    }
 
-    my $ret = shift( @{ $self->{_q} } );
+    my ( $retbase, $ret ) = @{ shift( @{ $self->{_q} } ) };
 
-    die if $ret ne int($ret);
-    die if $ret < 0;
-    die if $ret >= $base;
+    die                             if $retbase ne $base;
+    die                             if $ret ne int($ret);
+    die                             if $ret < 0;
+    Carp::confess("big $ret $base") if $ret >= $base;
 
     return $ret;
 }
@@ -772,3 +1016,4 @@ or distributed except according to the terms contained in the COPYING file.
 Copyright (c) 2020 Shlomi Fish
 
 =cut
+
